from typing import List
from BuildingElement import BuildingElement

import NemAll_Python_BasisElements as AllplanBasisElements
import NemAll_Python_BaseElements as AllplanBaseElements
import NemAll_Python_Geometry as AllplanGeo
import NemAll_Python_IFW_Input as AllplanIFW

from BuildingElementPaletteService import BuildingElementPaletteService
from BuildingElementService import BuildingElementService

def check_allplan_version(build_ele, version):
    """
    Check the current Allplan version

    Args:
        build_ele: the building element.
        version:   the current Allplan version

    Returns:
        True/False if version is supported by this script
    """

    # Delete unused arguments
    del build_ele
    del version

    # Support all versions
    return True

def create_interactor(coord_input: AllplanIFW.CoordinateInput, pyp_path: str, str_table_service):
    """
    Start interactor
    Args:
        coord_input: coordinate input
        pyp_path: path of the pyp file
        str_table_service: string table service
    """

    return StripFoundationClass(coord_input, pyp_path, str_table_service)

class StripFoundationClass():
    def __init__(self, coord_input: AllplanIFW.CoordinateInput, pyp_path: str, str_table_service):

        self.coord_input = coord_input
        self.str_table_service = str_table_service
        self.build_ele_service = BuildingElementService()

        #show the palette
        self.is_success = self.show_palette(pyp_path)
        if not self.is_success:
            return

        # Selected points
        self.selected_pnts = []

        # Show input message
        self.reset()

    def reset(self):
        """reset selection for next foundation"""
        self.selected_pnts.clear()
        self.coord_input.InitFirstPointInput(AllplanIFW.InputStringConvert("Select the first point to create foundation"))

    def show_palette(self, pyp_path) -> bool:
        """read data from pyp file and show the palette
           Return: bool for success
        """
        result, self.build_ele_script, self.build_ele_list, self.control_props_list,    \
            self.build_ele_composite, part_name, self.file_name = \
            self.build_ele_service.read_data_from_pyp(pyp_path + "\\StripFoundation.pyp", self.str_table_service.str_table, False, 
                                                      self.str_table_service.material_str_table)

        if not result:
            return False

        self.palette_service = BuildingElementPaletteService(self.build_ele_list, self.build_ele_composite,
                                                             self.build_ele_script,
                                                             self.control_props_list, self.file_name)

        self.palette_service.show_palette(part_name)

        return True

    def process_mouse_msg(self,
                          mouse_msg: int,
                          pnt      : AllplanGeo.Point2D,
                          msg_info : AllplanIFW.AddMsgInfo) -> bool:
        """
        Handles the process mouse message event

        Args:
            mouse_msg: the mouse message.
            pnt      : the input point.
            msg_info : additional message info.

        Returns:
            True/False for success.
        """

        # Get input point
        if self.selected_pnts:
            input_pnt = self.coord_input.GetInputPoint(mouse_msg, pnt, msg_info, self.selected_pnts[-1], True).GetPoint()
        else:
            input_pnt = self.coord_input.GetInputPoint(mouse_msg, pnt, msg_info).GetPoint()

        if self.coord_input.IsMouseMove(mouse_msg):
            return True

        # Save input point to selected points
        self.selected_pnts.append(input_pnt)

        # Create model and reset for next use
        if len(self.selected_pnts) == 2:
            #create model
            self.reset()

        # Message for second point
        if len(self.selected_pnts) == 1:
            self.coord_input.InitNextPointInput(AllplanIFW.InputStringConvert("Select the second point to create foundation"))

        return True

    def on_cancel_function(self) -> bool:
        """
        Cancel the input function

        Returns:
            True/False for success.
        """

        if self.is_success:
            self.palette_service.close_palette()
        return True

    def on_mouse_leave(self):
        """
        Handles the mouse leave event
        """
        pass

    def on_preview_draw(self):
        """
        Handles the preview draw event
        """
        pass

    def modify_element_property(self,
                                page : str,
                                name : str,
                                value):
        """
        Modify property of element

        Args:
            page:   the page of the property
            name:   the name of the property.
            value:  new value for property.
        """
        pass